//Project: LanderControl
//Homepage: www.HomoFaciens.de
//Author Norbert Heinz
//Version: 1.0
//Creation date: 05.06.2011
//Copyright: (c) 2011 by Norbert Heinz
//This program is free software you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation version 3 of the License.
//This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//You should have received a copy of the GNU General Public License along with this program if not, see http://www.gnu.org/licenses/
//
//Program code based on PowerSwitch (author Christian Starkjohann) - Thanks!
//Program uses generic shared VID/PID pair of OBJECTIVE DEVELOPMENT Software GmbH - see documentation for licence and details

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <usb.h>    /* this is libusb, see http://libusb.sourceforge.net/ */

#define USBDEV_SHARED_VENDOR    0x16C0  /* VOTI */
#define USBDEV_SHARED_PRODUCT   0x05DC  /* Obdev's free shared PID */
/* Use obdev's generic shared VID/PID pair and follow the rules outlined
 * in firmware/usbdrv/USBID-License.txt.
 */

#define RCCMD_ECHO              0
#define RCCMD_MODULENUMBER      1
#define RCCMD_INSTRUCTION       2
#define RCCMD_HORIZONTALANGLE   3
#define RCCMD_VERTICALANGLE     4
#define RCCMD_ARMHEIGHTANGLE    5
#define RCCMD_LIGHT             6
#define RCCMD_SLEEP            10
#define RCCMD_GETSTATUS        11
#define RCCMD_GETIRMODULE      12
/* These are the vendor specific SETUP commands implemented by our USB device */

static void usage(char *name)
{
    fprintf(stderr, "usage:\n");
    fprintf(stderr, "  %s test\n", name);
    fprintf(stderr, "  %s i <instruction>\n", name);
    fprintf(stderr, "  %s m <modulenumber>\n", name);
    fprintf(stderr, "  %s l <light>\n", name);
    fprintf(stderr, "  %s a <ArmHeightAngle>\n", name);
    fprintf(stderr, "  %s h <HorizontalAngle>\n", name);
    fprintf(stderr, "  %s v <VerticalAngle>\n", name);
    fprintf(stderr, "  %s g (get moduleNumber) \n", name);
    fprintf(stderr, "  %s s (sleep)\n", name);


    fprintf(stderr, "instruction is a value in the range 0...255\n");
    fprintf(stderr, "modulenumber is a value in the range 1...255\n");
    fprintf(stderr, "light is a value in the range 0...200\n");
    fprintf(stderr, "ArmHeightAngle is a value in the range 35...85 or 0\n");
    fprintf(stderr, "HorizontalAngle is a value in the range 50...110 or 0\n");
    fprintf(stderr, "VerticalAngle is a value in the range 35...110 or 0\n");
}


static int  usbGetStringAscii(usb_dev_handle *dev, int index, int langid, char *buf, int buflen)
{
char    buffer[256];
int     rval, i;

    if((rval = usb_control_msg(dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR, (USB_DT_STRING << 8) + index, langid, buffer, sizeof(buffer), 1000)) < 0)
        return rval;
    if(buffer[1] != USB_DT_STRING)
        return 0;
    if((unsigned char)buffer[0] < rval)
        rval = (unsigned char)buffer[0];
    rval /= 2;
    /* lossy conversion to ISO Latin1 */
    for(i=1;i<rval;i++){
        if(i > buflen)  /* destination buffer overflow */
            break;
        buf[i-1] = buffer[2 * i];
        if(buffer[2 * i + 1] != 0)  /* outside of ISO Latin1 range */
            buf[i-1] = '?';
    }
    buf[i-1] = 0;
    return i-1;
}

#define USB_ERROR_NOTFOUND  1
#define USB_ERROR_ACCESS    2
#define USB_ERROR_IO        3

static int usbOpenDevice(usb_dev_handle **device, int vendor, char *vendorName, int product, char *productName)
{
struct usb_bus      *bus;
struct usb_device   *dev;
usb_dev_handle      *handle = NULL;
int                 errorCode = USB_ERROR_NOTFOUND;
static int          didUsbInit = 0;

    if(!didUsbInit){
        didUsbInit = 1;
        usb_init();
    }
    usb_find_busses();
    usb_find_devices();
    for(bus=usb_get_busses(); bus; bus=bus->next){
        for(dev=bus->devices; dev; dev=dev->next){
            if(dev->descriptor.idVendor == vendor && dev->descriptor.idProduct == product){
                printf ("This is our device: Vendor=0x%x, Product=0x%x\n", dev->descriptor.idVendor, dev->descriptor.idProduct);
                char    string[256];
                int     len;
                handle = usb_open(dev); /* we need to open the device in order to query strings */
                if(!handle){
                    errorCode = USB_ERROR_ACCESS;
                    fprintf(stderr, "Warning: cannot open USB device: %s\n", usb_strerror());
                    continue;
                }
                if(vendorName == NULL && productName == NULL){  /* name does not matter */
                    break;
                }
                /* now check whether the names match: */
                len = usbGetStringAscii(handle, dev->descriptor.iManufacturer, 0x0409, string, sizeof(string));
                if(len < 0){
                    errorCode = USB_ERROR_IO;
                    fprintf(stderr, "Warning: cannot query manufacturer for device: %s\n", usb_strerror());
                }
                else{
                    errorCode = USB_ERROR_NOTFOUND;
                    if(strcmp(string, vendorName) == 0){
                        len = usbGetStringAscii(handle, dev->descriptor.iProduct, 0x0409, string, sizeof(string));
                        if(len < 0){
                            errorCode = USB_ERROR_IO;
                            fprintf(stderr, "Warning: cannot query product for device: %s\n", usb_strerror());
                        }else{
                            errorCode = USB_ERROR_NOTFOUND;
                            if(strcmp(string, productName) == 0)
                                break;
                        }
                    }
                }
                usb_close(handle);
                handle = NULL;
            }
            else{
              printf ("Not our device: Vendor=0x%x, Product=0x%x\n", dev->descriptor.idVendor, dev->descriptor.idProduct);
            }
        }
        if(handle)
            break;
    }
    if(handle != NULL){
        errorCode = 0;
        *device = handle;
    }
    return errorCode;
}


int main(int argc, char **argv)
{
usb_dev_handle      *handle = NULL;
unsigned char       buffer[8];
int                 nBytes;
int                 argsError = 0;
int                 modulenumber = 0;
int                 instruction = 0;
int                 light = 0;
int                 ArmHeightAngle = 0;
int                 VerticalAngle = 0;
int                 HorizontalAngle = 0;
int                 receiveByte = 0;

printf("argc=%d\n", argc);
    if(argc < 2){
        usage(argv[0]);
        exit(1);
    }
    if(argc > 2){
      if(strcmp(argv[1], "m") == 0){
        modulenumber = atoi(argv[2]);
        if(modulenumber < 0 || modulenumber > 255){
          argsError = 1;
        }
      }
      else if(strcmp(argv[1], "i") == 0){
        instruction = atoi(argv[2]);
        if(instruction < 0 || instruction > 255){
          argsError = 1;
        }
      }
      else if(strcmp(argv[1], "l") == 0){
        light = atoi(argv[2]);
        if(light < 0 || light > 200){
          argsError = 1;
        }
      }
      else if(strcmp(argv[1], "a") == 0){
        ArmHeightAngle = atoi(argv[2]);
        if(ArmHeightAngle < 35 || ArmHeightAngle > 85){
          if(ArmHeightAngle != 0){
            argsError = 1;
          }
        }
      }
      else if(strcmp(argv[1], "h") == 0){
        HorizontalAngle = atoi(argv[2]);
        if(HorizontalAngle < 50 || HorizontalAngle > 110){
          if(HorizontalAngle != 0){
            argsError = 1;
          }
        }
      }
      else if(strcmp(argv[1], "v") == 0){
        VerticalAngle = atoi(argv[2]);
        if(VerticalAngle < 35 || VerticalAngle > 110){
          if(VerticalAngle != 0){
            argsError = 1;
          }
        }
      }
    }//if(argc > 2){

    if(argsError == 1){
      usage(argv[0]);
      exit(1);
    }

    usb_init();
    if(usbOpenDevice(&handle, USBDEV_SHARED_VENDOR, "www.HomoFaciens.de", USBDEV_SHARED_PRODUCT, "LanderControl") != 0){
        fprintf(stderr, "Could not find USB device \"LanderControl\" with vid=0x%x pid=0x%x\n", USBDEV_SHARED_VENDOR, USBDEV_SHARED_PRODUCT);
        exit(1);
    }
/* We have searched all devices on all busses for our USB device above. Now
 * try to open it and perform the vendor specific control operations for the
 * function requested by the user.
 */
    if(strcmp(argv[1], "test") == 0){
        int i, v, r;
/* The test consists of writing 1000 random numbers to the device and checking
 * the echo. This should discover systematic bit errors (e.g. in bit stuffing).
 */
        for(i=0;i<1000;i++){
            v = rand() & 0xffff;
            nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, RCCMD_ECHO, v, 0, (char *)buffer, sizeof(buffer), 5000);
            if(nBytes < 2){
                if(nBytes < 0)
                    fprintf(stderr, "USB error: %s\n", usb_strerror());
                fprintf(stderr, "only %d bytes received in iteration %d\n", nBytes, i);
                fprintf(stderr, "value sent = 0x%x\n", v);
                exit(1);
            }
            r = buffer[0] | (buffer[1] << 8);
            if(r != v){
                fprintf(stderr, "data error: received 0x%x instead of 0x%x in iteration %d\n", r, v, i);
                exit(1);
            }
        }
        printf("test succeeded\n");
    }
    else if(strcmp(argv[1], "g") == 0){
      nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, RCCMD_GETSTATUS, receiveByte, 0, (char *)buffer, sizeof(buffer), 8);
      if(nBytes < 2){
        if(nBytes < 0)
        fprintf(stderr, "USB error: %s\n", usb_strerror());
        exit(1);
      }
      printf("get01=%d, get02=%d\n", buffer[0], buffer[1]);
    }
    else if(strcmp(argv[1], "r") == 0){
      nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, RCCMD_GETIRMODULE, receiveByte, 0, (char *)buffer, sizeof(buffer), 8);
      if(nBytes < 2){
        if(nBytes < 0)
        fprintf(stderr, "USB error: %s\n", usb_strerror());
        exit(1);
      }
      printf("IR module number=%d, SendStatus=%d\n", buffer[0], buffer[1]);
    }
    else if(strcmp(argv[1], "m") == 0){
      printf ("Calling module number %d.\n", modulenumber);
      nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, RCCMD_MODULENUMBER, modulenumber, 0, (char *)buffer, sizeof(buffer), 8);
      if(nBytes < 0){
          fprintf(stderr, "USB error: %s\n", usb_strerror());
      }
      printf("Set ModuleNumber to %d\n", modulenumber);
    }
    else if(strcmp(argv[1], "l") == 0){
      nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, RCCMD_LIGHT, light, 0, (char *)buffer, sizeof(buffer), 8);
      if(nBytes < 0){
          fprintf(stderr, "USB error: %s\n", usb_strerror());
      }
      printf("Set light to %d\n", light);
    }
    else if(strcmp(argv[1], "a") == 0){
      nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, RCCMD_ARMHEIGHTANGLE, ArmHeightAngle, 0, (char *)buffer, sizeof(buffer), 8);
      if(nBytes < 0){
          fprintf(stderr, "USB error: %s\n", usb_strerror());
      }
      printf("Set ArmHeightAngle to %d\n", ArmHeightAngle);
    }
    else if(strcmp(argv[1], "h") == 0){
      nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, RCCMD_HORIZONTALANGLE, HorizontalAngle, 0, (char *)buffer, sizeof(buffer), 8);
      if(nBytes < 0){
          fprintf(stderr, "USB error: %s\n", usb_strerror());
      }
      printf("Set HorizontalAngle to %d\n", HorizontalAngle);
    }
    else if(strcmp(argv[1], "v") == 0){
      nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, RCCMD_VERTICALANGLE, VerticalAngle, 0, (char *)buffer, sizeof(buffer), 8);
      if(nBytes < 0){
          fprintf(stderr, "USB error: %s\n", usb_strerror());
      }
      printf("Set VerticalAngle to %d\n", VerticalAngle);
    }
    else if(strcmp(argv[1], "s") == 0){
      nBytes = usb_control_msg(handle, USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, RCCMD_SLEEP, VerticalAngle, 0, (char *)buffer, sizeof(buffer), 8);
      if(nBytes < 0){
          fprintf(stderr, "USB error: %s\n", usb_strerror());
      }
      printf("Set Lander to sleep mode\n");
    }

    usb_close(handle);
    return 0;
}
